@@ -1,10 +1,10 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (beto@coresecurity.com, bethus@gmail.com)
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   Helper functions for kerberos
@@ -17,19 +17,24 @@ import random
 import socket
 import struct
 import os
+
 from pyasn1.codec.der import decoder, encoder
-from impacket.krb5.asn1 import AS_REQ, AP_REQ, TGS_REQ, KERB_PA_PAC_REQUEST, KRB_ERROR, PA_ENC_TS_ENC, METHOD_DATA, AS_REP, TGS_REP, EncryptedData, Authenticator, EncASRepPart, EncTGSRepPart, seq_append, seq_set, seq_set_iter, seq_set_dict, KERB_ERROR_DATA, METHOD_DATA, ETYPE_INFO2_ENTRY, ETYPE_INFO_ENTRY, ETYPE_INFO2, ETYPE_INFO, AP_REP, EncAPRepPart
+from binascii import unhexlify
+
+from impacket.krb5.asn1 import AS_REQ, AP_REQ, TGS_REQ, KERB_PA_PAC_REQUEST, KRB_ERROR, PA_ENC_TS_ENC, AS_REP, TGS_REP, \
+    EncryptedData, Authenticator, EncASRepPart, EncTGSRepPart, seq_set, seq_set_iter, KERB_ERROR_DATA, METHOD_DATA, \
+    ETYPE_INFO2, ETYPE_INFO, AP_REP, EncAPRepPart
 from impacket.krb5.types import KerberosTime, Principal, Ticket
-from impacket.krb5.gssapi import CheckSumField, GSS_C_DCE_STYLE, GSS_C_DELEG_FLAG, GSS_C_MUTUAL_FLAG, GSS_C_REPLAY_FLAG, GSS_C_SEQUENCE_FLAG, GSS_C_CONF_FLAG, GSS_C_INTEG_FLAG 
+from impacket.krb5.gssapi import CheckSumField, GSS_C_DCE_STYLE, GSS_C_MUTUAL_FLAG, GSS_C_REPLAY_FLAG, \
+    GSS_C_SEQUENCE_FLAG, GSS_C_CONF_FLAG, GSS_C_INTEG_FLAG
 from impacket.krb5 import constants
-from impacket.krb5.crypto import _RC4, Key, _enctype_table
+from impacket.krb5.crypto import Key, _enctype_table
 from impacket.smbconnection import SessionError
 from impacket.spnego import SPNEGO_NegTokenInit, TypesMech, SPNEGO_NegTokenResp
-from impacket.winregistry import hexdump
 from impacket import nt_errors, LOG
-from impacket.structure import Structure
 from impacket.krb5.ccache import CCache
 
+
 def sendReceive(data, host, kdcHost):
     if kdcHost is None:
         targetHost = host
@@ -38,6 +43,7 @@ def sendReceive(data, host, kdcHost):
 
     messageLen = struct.pack('!i', len(data))
 
+    LOG.debug('Trying to connect to KDC at %s' % targetHost)
     s = socket.socket()
     s.connect((targetHost, 88))
     s.sendall(messageLen + data)
@@ -180,7 +186,7 @@ def getKerberosTGT(clientName, password, domain, lmhash, nthash, aesKey='', kdcH
     if nthash != '':
         key = Key(cipher.enctype, nthash)
     elif aesKey != '':
-        key = Key(cipher.enctype, aesKey.decode('hex'))
+        key = Key(cipher.enctype, unhexlify(aesKey))
     else:
         key = cipher.string_to_key(password, encryptionTypesData[enctype], None)
     encodedTimeStamp = encoder.encode(timeStamp)
@@ -330,7 +336,7 @@ def getKerberosTGS(serverName, domain, kdcHost, tgt, cipher, sessionKey):
 
     reqBody['kdc-options'] = constants.encodeFlags(opts)
     seq_set(reqBody, 'sname', serverName.components_to_asn1)
-    reqBody['realm'] = str(decodedTGT['crealm'])
+    reqBody['realm'] = domain
 
     now = datetime.datetime.utcnow() + datetime.timedelta(days=1)
 
@@ -358,6 +364,19 @@ def getKerberosTGS(serverName, domain, kdcHost, tgt, cipher, sessionKey):
     encTGSRepPart = decoder.decode(plainText, asn1Spec = EncTGSRepPart())[0]
 
     newSessionKey = Key(cipher.enctype, str(encTGSRepPart['key']['keyvalue']))
+
+    # Check we've got what we asked for
+    res = decoder.decode(r, asn1Spec = TGS_REP())[0]
+    spn = Principal()
+    spn.from_asn1(res['ticket'], 'realm', 'sname')
+
+    if spn.components[0] == serverName.components[0]:
+        # Yes.. bye bye
+        return r, cipher, sessionKey, newSessionKey
+    else:
+        # Let's extract the Ticket, change the domain and keep asking
+        domain = spn.components[1]
+        return getKerberosTGS(serverName, domain, kdcHost, r, cipher, newSessionKey)
     
     return r, cipher, sessionKey, newSessionKey
 
@@ -421,6 +440,11 @@ def getKerberosType1(username, password, domain, lmhash, nthash, aesKey='', TGT
                 # No cache present
                 pass
             else:
+                # retrieve user and domain information from CCache file if needed
+                if username == '' and len(ccache.principal.components) > 0:
+                    username = ccache.principal.components[0]['data']
+                if domain == '':
+                    domain = ccache.principal.realm['data']
                 LOG.debug("Using Kerberos Cache: %s" % os.getenv('KRB5CCNAME'))
                 principal = 'host/%s@%s' % (targetName.upper(), domain.upper())
                 creds = ccache.getCredential(principal)
@@ -451,7 +475,8 @@ def getKerberosType1(username, password, domain, lmhash, nthash, aesKey='', TGT
                         # done, byebye.
                         if lmhash is '' and nthash is '' and (aesKey is '' or aesKey is None) and TGT is None and TGS is None:
                             from impacket.ntlm import compute_lmhash, compute_nthash
-                            lmhash = compute_lmhash(password) 
+                            LOG.debug('Got KDC_ERR_ETYPE_NOSUPP, fallback to RC4')
+                            lmhash = compute_lmhash(password)
                             nthash = compute_nthash(password) 
                             continue
                         else:
@@ -467,7 +492,7 @@ def getKerberosType1(username, password, domain, lmhash, nthash, aesKey='', TGT
         # Now that we have the TGT, we should ask for a TGS for cifs
 
         if TGS is None:
-            serverName = Principal('host/%s' % (targetName), type=constants.PrincipalNameType.NT_SRV_INST.value)
+            serverName = Principal('host/%s' % targetName, type=constants.PrincipalNameType.NT_SRV_INST.value)
             try:
                 tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(serverName, domain, kdcHost, tgt, cipher, sessionKey)
             except KerberosError, e:
@@ -478,7 +503,8 @@ def getKerberosType1(username, password, domain, lmhash, nthash, aesKey='', TGT
                     # done, byebye.
                     if lmhash is '' and nthash is '' and (aesKey is '' or aesKey is None) and TGT is None and TGS is None:
                         from impacket.ntlm import compute_lmhash, compute_nthash
-                        lmhash = compute_lmhash(password) 
+                        LOG.debug('Got KDC_ERR_ETYPE_NOSUPP, fallback to RC4')
+                        lmhash = compute_lmhash(password)
                         nthash = compute_nthash(password) 
                     else:
                         raise 
