@@ -1,10 +1,10 @@
-# Copyright (c) 2003-2012 CORE Security Technologies)
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (beto@coresecurity.com)
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-SMB2] Protocol Implementation (SMB2 and SMB3)
@@ -25,14 +25,21 @@
 # [ ] Fix up all the 'ToDo' comments inside the code
 #
 
-import socket, string, ntpath
+import socket
+import ntpath
 import random
-from impacket import nmb, smb3structs, nt_errors, spnego, ntlm, uuid, crypto, LOG
-from impacket.smb3structs import *
-from impacket.nt_errors import *
-from impacket.spnego import *
+import string
+import struct
 from binascii import a2b_hex
 from contextlib import contextmanager
+
+from impacket import nmb, ntlm, uuid, crypto, LOG
+from impacket.smb3structs import *
+from impacket.nt_errors import STATUS_SUCCESS, STATUS_MORE_PROCESSING_REQUIRED, STATUS_INVALID_PARAMETER, \
+    STATUS_NO_MORE_FILES, STATUS_PENDING, STATUS_NOT_IMPLEMENTED, ERROR_MESSAGES
+from impacket.spnego import SPNEGO_NegTokenInit, TypesMech, SPNEGO_NegTokenResp
+
+
 # For signing
 import hashlib, hmac, copy
 
@@ -187,7 +194,16 @@ class SMB3:
         self._timeout = timeout
         self._Connection['ServerIP'] = remote_host
         self._NetBIOSSession = None
-        
+
+        self.__userName = ''
+        self.__password = ''
+        self.__domain   = ''
+        self.__lmhash   = ''
+        self.__nthash   = ''
+        self.__kdc      = ''
+        self.__aesKey   = ''
+        self.__TGT      = None
+        self.__TGS      = None
 
         if sess_port == 445 and remote_name == '*SMBSERVER':
            self._Connection['ServerName'] = remote_host
@@ -238,6 +254,15 @@ class SMB3:
     def getServerOS(self):
         return self._Session['ServerOS']
 
+    def getServerOSMajor(self):
+        return self._Session['ServerOSMajor']
+
+    def getServerOSMinor(self):
+        return self._Session['ServerOSMinor']
+
+    def getServerOSBuild(self):
+        return self._Session['ServerOSBuild']
+
     def isGuestSession(self):
         return self._Session['SessionFlags'] & SMB2_SESSION_FLAG_IS_GUEST 
 
@@ -246,7 +271,7 @@ class SMB3:
 
     @contextmanager
     def useTimeout(self, timeout):
-        prev_timeout = self.setTimeout(timeout)
+        prev_timeout = self.getTimeout(timeout)
         try:
             yield
         finally:
@@ -358,7 +383,6 @@ class SMB3:
                 if data.get_trailer().startswith('\xfeSMB'):
                     packet = SMB2Packet(data.get_trailer())
                 else:
-                    packet = SMB2_TRANSFORM_HEADER(data.get_trailer())
                     # Packet is encrypted
                     transformHeader = SMB2_TRANSFORM_HEADER(data.get_trailer())
                     from Crypto.Cipher import AES
@@ -459,7 +483,7 @@ class SMB3:
         # TGS['cipher'] = the cipher used
         # TGS['sessionKey'] = the sessionKey
         # If we have hashes, normalize them
-        if ( lmhash != '' or nthash != ''):
+        if lmhash != '' or nthash != '':
             if len(lmhash) % 2:     lmhash = '0%s' % lmhash
             if len(nthash) % 2:     nthash = '0%s' % nthash
             try: # just in case they were converted already
@@ -488,11 +512,10 @@ class SMB3:
         #sessionSetup['Capabilities'] = SMB2_GLOBAL_CAP_LARGE_MTU | SMB2_GLOBAL_CAP_LEASING | SMB2_GLOBAL_CAP_DFS
 
         # Importing down here so pyasn1 is not required if kerberos is not used.
-        from impacket.krb5.asn1 import AP_REQ, Authenticator, TGS_REP, seq_set, seq_set_dict
+        from impacket.krb5.asn1 import AP_REQ, Authenticator, TGS_REP, seq_set
         from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS
         from impacket.krb5 import constants
         from impacket.krb5.types import Principal, KerberosTime, Ticket
-        from impacket.winregistry import hexdump
         from pyasn1.codec.der import decoder, encoder
         import datetime
 
@@ -588,7 +611,6 @@ class SMB3:
             self._Session['SigningRequired'] = self._Connection['RequireSigning']
             self._Session['UserCredentials'] = (user, password, domain, lmhash, nthash)
             self._Session['Connection']      = self._NetBIOSSession.get_socket()
-            sessionSetupResponse = SMB2SessionSetup_Response(ans['Data'])
 
             self._Session['SessionKey']  = sessionKey.contents[:16]
             if self._Session['SigningRequired'] is True and self._Connection['Dialect'] == SMB2_DIALECT_30:
@@ -618,7 +640,7 @@ class SMB3:
 
     def login(self, user, password, domain = '', lmhash = '', nthash = ''):
         # If we have hashes, normalize them
-        if ( lmhash != '' or nthash != ''):
+        if lmhash != '' or nthash != '':
             if len(lmhash) % 2:     lmhash = '0%s' % lmhash
             if len(nthash) % 2:     nthash = '0%s' % nthash
             try: # just in case they were converted already
@@ -652,7 +674,7 @@ class SMB3:
 
         # NTLMSSP
         blob['MechTypes'] = [TypesMech['NTLMSSP - Microsoft NTLM Security Support Provider']]
-        auth = ntlm.getNTLMSSPType1('',domain, self._Connection['RequireSigning'])
+        auth = ntlm.getNTLMSSPType1('','', self._Connection['RequireSigning'])
         blob['MechToken'] = str(auth)
 
         sessionSetup['SecurityBufferLength'] = len(blob)
@@ -683,8 +705,7 @@ class SMB3:
             # Let's parse some data and keep it to ourselves in case it is asked
             ntlmChallenge = ntlm.NTLMAuthChallenge(respToken['ResponseToken'])
             if ntlmChallenge['TargetInfoFields_len'] > 0:
-                infoFields = ntlmChallenge['TargetInfoFields']
-                av_pairs = ntlm.AV_PAIRS(ntlmChallenge['TargetInfoFields'][:ntlmChallenge['TargetInfoFields_len']]) 
+                av_pairs = ntlm.AV_PAIRS(ntlmChallenge['TargetInfoFields'][:ntlmChallenge['TargetInfoFields_len']])
                 if av_pairs[ntlm.NTLMSSP_AV_HOSTNAME] is not None:
                    try:
                        self._Session['ServerName'] = av_pairs[ntlm.NTLMSSP_AV_HOSTNAME][1].decode('utf-16le')
@@ -708,8 +729,12 @@ class SMB3:
                 # Parse Version to know the target Operating system name. Not provided elsewhere anymore
                 if ntlmChallenge.fields.has_key('Version'):
                     version = ntlmChallenge['Version']
+
                     if len(version) >= 4:
                         self._Session['ServerOS'] = "Windows %d.%d Build %d" % (ord(version[0]), ord(version[1]), struct.unpack('<H',version[2:4])[0])
+                        self._Session["ServerOSMajor"] = ord(version[0])
+                        self._Session["ServerOSMinor"] = ord(version[1])
+                        self._Session["ServerOSBuild"] = struct.unpack('<H',version[2:4])[0]
 
             type3, exportedSessionKey = ntlm.getNTLMSSPType3(auth, respToken['ResponseToken'], user, password, domain, lmhash, nthash)
    
@@ -861,7 +886,7 @@ class SMB3:
            if len(fileName.split('\\')) > 2:
                parentDir = ntpath.dirname(pathName)
            if self.GlobalFileTable.has_key(parentDir):
-               log.critical("Don't know what to do now! :-o")
+               LOG.critical("Don't know what to do now! :-o")
                raise
            else:
                parentEntry = copy.deepcopy(FILE)
@@ -893,6 +918,9 @@ class SMB3:
 
         if createContexts is not None:
             smb2Create['Buffer'] += createContexts
+            # Offset = SMB2_HEADER (64) + CREATE_REQUEST_HEADER (56) + NameLength
+            smb2Create['CreateContextsOffset'] = 64 + 56 + smb2Create['NameLength']
+            smb2Create['CreateContextsLength'] = len(createContexts)
         else:
             smb2Create['CreateContextsOffset'] = 0
             smb2Create['CreateContextsLength'] = 0
@@ -1031,7 +1059,7 @@ class SMB3:
         packet['Data'] = smbWrite
 
         packetID = self.sendSMB(packet)
-        if waitAnswer == True:
+        if waitAnswer is True:
             ans = self.recvSMB(packetID)
         else:
             return maxBytesToWrite
@@ -1094,7 +1122,7 @@ class SMB3:
         smbCancel = SMB2Cancel()
 
         packet['Data']      = smbCancel
-        packetID = self.sendSMB(packet)
+        self.sendSMB(packet)
 
     def ioctl(self, treeId, fileId = None, ctlCode = -1, flags = 0, inputBlob = '',  maxInputResponse = None, maxOutputResponse = None, waitAnswer = 1):
         if self._Session['TreeConnectTable'].has_key(treeId) is False:
@@ -1156,7 +1184,6 @@ class SMB3:
         ans = self.recvSMB(packetID)
 
         if ans.isValidAnswer(STATUS_SUCCESS):
-            smbFlushResponse = SMB2Flush_Response(ans['Data'])
             return True
 
     def lock(self, treeId, fileId, locks, lockSequence = 0):
@@ -1210,6 +1237,15 @@ class SMB3:
         ans = self.recvSMB(packetID)
 
         if ans.isValidAnswer(STATUS_SUCCESS):
+            # We clean the stuff we used in case we want to authenticate again
+            # within the same connection
+            self._Session['UserCredentials']   = ''
+            self._Session['Connection']        = 0
+            self._Session['SessionID']         = 0
+            self._Session['SigningRequired']   = False
+            self._Session['SigningKey']        = ''
+            self._Session['SessionKey']        = ''
+            self._Session['SigningActivated']  = False
             return True
 
     def queryInfo(self, treeId, fileId, inputBlob = '', infoType = SMB2_0_INFO_FILE, fileInfoClass = SMB2_FILE_STANDARD_INFO, additionalInformation = 0, flags = 0 ):
@@ -1391,7 +1427,7 @@ class SMB3:
 
         fileId = None
         try:
-            fileId = self.create(treeId, pathName,GENERIC_ALL | DELETE, FILE_SHARE_READ | FILE_SHARE_WRITE |FILE_SHARE_DELETE, FILE_DIRECTORY_FILE | FILE_DELETE_ON_CLOSE, FILE_OPEN, 0)          
+            fileId = self.create(treeId, pathName, DELETE, FILE_SHARE_DELETE, FILE_DIRECTORY_FILE | FILE_DELETE_ON_CLOSE, FILE_OPEN, 0)
         finally:
             if fileId is not None:
                 self.close(treeId, fileId)
@@ -1410,7 +1446,7 @@ class SMB3:
 
         fileId = None
         try:
-            fileId = self.create(treeId, pathName,GENERIC_ALL | DELETE, FILE_SHARE_READ | FILE_SHARE_WRITE |FILE_SHARE_DELETE, FILE_NON_DIRECTORY_FILE | FILE_DELETE_ON_CLOSE, FILE_OPEN, 0)          
+            fileId = self.create(treeId, pathName,DELETE | FILE_READ_ATTRIBUTES, FILE_SHARE_DELETE, FILE_NON_DIRECTORY_FILE | FILE_DELETE_ON_CLOSE, FILE_OPEN, 0)
         finally:
             if fileId is not None:
                 self.close(treeId, fileId)
@@ -1441,7 +1477,7 @@ class SMB3:
             else:
                 written = 0
                 toBeRead = fileSize-offset
-                while written < (toBeRead):
+                while written < toBeRead:
                     data = self.read(treeId, fileId, offset, self._Connection['MaxReadSize'])
                     written += len(data)
                     offset  += len(data)
@@ -1490,7 +1526,12 @@ class SMB3:
 
         return self.ioctl(treeId, None, FSCTL_PIPE_WAIT,flags=SMB2_0_IOCTL_IS_FSCTL, inputBlob=pipeWait, maxInputResponse = 0, maxOutputResponse=0)
         
+    def getIOCapabilities(self):
+        res = dict()
 
+        res['MaxReadSize'] = self._Connection['MaxReadSize']
+        res['MaxWriteSize'] = self._Connection['MaxWriteSize']
+        return res
         
 
     ######################################################################
@@ -1503,6 +1544,9 @@ class SMB3:
     get_remote_name            = getServerName
     get_remote_host            = getServerIP
     get_server_os              = getServerOS
+    get_server_os_major        = getServerOSMajor
+    get_server_os_minor        = getServerOSMinor
+    get_server_os_build        = getServerOSBuild
     tree_connect_andx          = connectTree
     tree_connect               = connectTree
     connect_tree               = connectTree
@@ -1526,6 +1570,9 @@ class SMB3:
         # Always true :P 
         return True
 
+    def is_signing_required(self):
+        return self._Session["SigningRequired"] 
+
     def nt_create_andx(self, treeId, fileName, smb_packet=None, cmd = None):
         if len(fileName) > 0 and fileName[0] == '\\':
             fileName = fileName[1:]
