@@ -1,10 +1,10 @@
-# Copyright (c) 2003-2012 CORE Security Technologies)
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (beto@coresecurity.com)
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #
@@ -16,15 +16,17 @@
 import ntpath
 import string
 import socket
+
 from impacket import smb, smb3, nmb, nt_errors, LOG
 from smb3structs import *
 
+
 # So the user doesn't need to import smb, the smb3 are already in here
 SMB_DIALECT = smb.SMB_DIALECT
 
-class SMBConnection():
+class SMBConnection:
     """
-    SMBConnection class - beto
+    SMBConnection class
 
     :param string remoteName: name of the remote host, can be it's NETBIOS name, IP or *\*SMBSERVER*.  If the later, and port is 139, the library will try to get the target's server name.
     :param string remoteHost: target server's remote address (IPv4, IPv6)
@@ -32,15 +34,20 @@ class SMBConnection():
     :param integer/optional sess_port: target port to connect
     :param integer/optional timeout: timeout in seconds when receiving packets
     :param optional preferredDialect: the dialect desired to talk with the target server. If not specified the highest one available will be used
+    :param optional boolean manualNegotiate: the user manually performs SMB_COM_NEGOTIATE
 
     :return: a SMBConnection instance, if not raises a SessionError exception
     """
-    def __init__(self, remoteName='', remoteHost='', myName = None, sess_port = 445, timeout=60, preferredDialect = None, existingConnection = None):
+    def __init__(self, remoteName='', remoteHost='', myName = None, sess_port = 445, timeout=60, preferredDialect = None, existingConnection = None, manualNegotiate = False):
 
         self._SMBConnection = 0
         self._dialect       = ''
         self._nmbSession    = 0
-        hostType = nmb.TYPE_SERVER
+        self._sess_port     = sess_port
+        self._myName        = myName
+        self._remoteHost    = remoteHost
+        self._remoteName    = remoteName
+        self._timeout       = timeout
 
         if existingConnection is not None:
             # Existing Connection must be a smb or smb3 instance
@@ -49,25 +56,49 @@ class SMBConnection():
             return
 
         ##preferredDialect = smb.SMB_DIALECT
+
+        if manualNegotiate is False:
+            self.negotiateSession(preferredDialect)
+
+    def negotiateSession(self, preferredDialect=None, flags1=smb.SMB.FLAGS1_PATHCASELESS | smb.SMB.FLAGS1_CANONICALIZED_PATHS, flags2=smb.SMB.FLAGS2_EXTENDED_SECURITY | smb.SMB.FLAGS2_NT_STATUS | smb.SMB.FLAGS2_LONG_NAMES,
+                         negoData='\x02NT LM 0.12\x00\x02SMB 2.002\x00\x02SMB 2.???\x00'):
+        """
+        Perform protocol negotiation
+
+        :param string preferredDialect: the dialect desired to talk with the target server. If None is specified the highest one available will be used
+        :param string flags1: the SMB FLAGS capabilities
+        :param string flags2: the SMB FLAGS2 capabilities
+        :param string negoData: data to be sent as part of the nego handshake
+
+        :return: True, raises a Session Error if error.
+        """
+        hostType = nmb.TYPE_SERVER
         if preferredDialect is None:
             # If no preferredDialect sent, we try the highest available one.
-            packet = self._negotiateSession(myName, remoteName, remoteHost, sess_port, timeout)
+            packet = self._negotiateSession(self._myName, self._remoteName, self._remoteHost, self._sess_port, self._timeout, True, flags1=flags1, flags2=flags2, data=negoData)
             if packet[0] == '\xfe':
                 # Answer is SMB2 packet
-                self._SMBConnection = smb3.SMB3(remoteName, remoteHost, myName, hostType, sess_port, timeout, session = self._nmbSession )
+                self._SMBConnection = smb3.SMB3(self._remoteName, self._remoteHost, self._myName, hostType, self._sess_port, self._timeout, session = self._nmbSession )
             else:
                 # Answer is SMB packet, sticking to SMBv1
-                self._SMBConnection = smb.SMB(remoteName, remoteHost, myName, hostType, sess_port, timeout, session = self._nmbSession, negPacket = packet)
+                self._SMBConnection = smb.SMB(self._remoteName, self._remoteHost, self._myName, hostType, self._sess_port, self._timeout, session = self._nmbSession, negPacket = packet)
         else:
             if preferredDialect == smb.SMB_DIALECT:
-                self._SMBConnection = smb.SMB(remoteName, remoteHost, myName, hostType, sess_port, timeout)
+                self._SMBConnection = smb.SMB(self._remoteName, self._remoteHost, self._myName, hostType, self._sess_port, self._timeout)
             elif preferredDialect in [SMB2_DIALECT_002, SMB2_DIALECT_21, SMB2_DIALECT_30]:
-                self._SMBConnection = smb3.SMB3(remoteName, remoteHost, myName, hostType, sess_port, timeout, preferredDialect = preferredDialect)
+                self._SMBConnection = smb3.SMB3(self._remoteName, self._remoteHost, self._myName, hostType, self._sess_port, self._timeout, preferredDialect = preferredDialect)
             else:
                 LOG.critical("Unknown dialect ", preferredDialect)
                 raise
 
-    def _negotiateSession(self, myName, remoteName, remoteHost, sess_port, timeout, extended_security = True):
+        # propagate flags to the smb sub-object
+        # does not affect smb3 objects
+        if isinstance(self._SMBConnection, smb.SMB):
+            self._SMBConnection.set_flags(flags1=flags1, flags2=flags2)
+
+        return True
+
+    def _negotiateSession(self, myName, remoteName, remoteHost, sess_port, timeout, extended_security = True, flags1=0, flags2=0, data=None):
         # Here we follow [MS-SMB2] negotiation handshake trying to understand what dialects
         # (including SMB1) is supported on the other end.
 
@@ -82,19 +113,36 @@ class SMBConnection():
         if sess_port == 445 and remoteName == '*SMBSERVER':
            remoteName = remoteHost
 
-        self._nmbSession = nmb.NetBIOSTCPSession(myName, remoteName, remoteHost, nmb.TYPE_SERVER, sess_port, timeout)
-
+        tries = 0
         smbp = smb.NewSMBPacket()
-        negSession = smb.SMBCommand(smb.SMB.SMB_COM_NEGOTIATE)
-        if extended_security == True:
-            smbp['Flags2']=smb.SMB.FLAGS2_EXTENDED_SECURITY
-        negSession['Data'] = '\x02NT LM 0.12\x00\x02SMB 2.002\x00\x02SMB 2.???\x00'
-        smbp.addCommand(negSession)
-        self._nmbSession.send_packet(str(smbp))
+        smbp['Flags1'] = flags1
+        smbp['Flags2'] = flags2
+        resp = None
+        while tries < 2:
+            self._nmbSession = nmb.NetBIOSTCPSession(myName, remoteName, remoteHost, nmb.TYPE_SERVER, sess_port, timeout)
 
-        r = self._nmbSession.recv_packet(timeout)
+            negSession = smb.SMBCommand(smb.SMB.SMB_COM_NEGOTIATE)
+            if extended_security is True:
+                smbp['Flags2'] |= smb.SMB.FLAGS2_EXTENDED_SECURITY
+            negSession['Data'] = data
+            smbp.addCommand(negSession)
+            self._nmbSession.send_packet(str(smbp))
 
-        return r.get_trailer()
+            try:
+                resp = self._nmbSession.recv_packet(timeout)
+                break
+            except nmb.NetBIOSError:
+                # OSX Yosemite asks for more Flags. Let's give it a try and see what happens
+                smbp['Flags2'] |= smb.SMB.FLAGS2_NT_STATUS | smb.SMB.FLAGS2_LONG_NAMES | smb.SMB.FLAGS2_UNICODE
+                smbp['Data'] = []
+
+            tries += 1
+
+        if resp is None:
+            # No luck, quitting
+            raise
+
+        return resp.get_trailer()
 
 
     def getSMBServer(self):
@@ -121,16 +169,31 @@ class SMBConnection():
     def getServerOS(self):
         return self._SMBConnection.get_server_os()
 
+    def getServerOSMajor(self):
+        return self._SMBConnection.get_server_os_major()
+
+    def getServerOSMinor(self):
+        return self._SMBConnection.get_server_os_minor()
+
+    def getServerOSBuild(self):
+        return self._SMBConnection.get_server_os_build()
+
     def doesSupportNTLMv2(self):
         return self._SMBConnection.doesSupportNTLMv2()
 
     def isLoginRequired(self):
         return self._SMBConnection.is_login_required()
 
+    def isSigningRequired(self):
+        return self._SMBConnection.is_signing_required()
+
     def getCredentials(self):
         return self._SMBConnection.getCredentials()
 
-    def login(self, user, password, domain = '', lmhash = '', nthash = ''):
+    def getIOCapabilities(self):
+        return self._SMBConnection.getIOCapabilities()
+
+    def login(self, user, password, domain = '', lmhash = '', nthash = '', ntlmFallback = True):
         """
         logins into the target system
 
@@ -139,11 +202,15 @@ class SMBConnection():
         :param string domain: domain where the account is valid for
         :param string lmhash: LMHASH used to authenticate using hashes (password is not used)
         :param string nthash: NTHASH used to authenticate using hashes (password is not used)
+        :param bool ntlmFallback: If True it will try NTLMv1 authentication if NTLMv2 fails. Only available for SMBv1
 
         :return: None, raises a Session Error if error.
         """
-        try: 
-            return self._SMBConnection.login(user, password, domain, lmhash, nthash)
+        try:
+            if self.getDialect() == smb.SMB_DIALECT:
+                return self._SMBConnection.login(user, password, domain, lmhash, nthash, ntlmFallback)
+            else:
+                return self._SMBConnection.login(user, password, domain, lmhash, nthash)
         except (smb.SessionError, smb3.SessionError), e:
             raise SessionError(e.get_error_code())
 
@@ -217,8 +284,6 @@ class SMBConnection():
                         raise e
                 else:
                     raise e
-            else:
-                break
 
     def isGuestSession(self):
         try:
@@ -235,8 +300,11 @@ class SMBConnection():
 
     def connectTree(self,share):
         if self.getDialect() == smb.SMB_DIALECT:
-            share = ntpath.basename(share)
-            share = '\\\\' + self.getRemoteHost() + '\\' + share
+            # If we already have a UNC we do nothing.
+            if ntpath.ismount(share) is False:
+                # Else we build it
+                share = ntpath.basename(share)
+                share = '\\\\' + self.getRemoteHost() + '\\' + share
         try:
             return self._SMBConnection.connect_tree(share)
         except (smb.SessionError, smb3.SessionError), e:
@@ -257,7 +325,7 @@ class SMBConnection():
         :return: a list containing dict entries for each share, raises exception if error
         """
         # Get the shares through RPC
-        from impacket.dcerpc.v5 import transport, rpcrt, srvs
+        from impacket.dcerpc.v5 import transport, srvs
         rpctransport = transport.SMBTransport(self.getRemoteHost(), self.getRemoteHost(), filename = r'\srvsvc', smb_connection = self)
         dce = rpctransport.get_dce_rpc()
         dce.connect()
@@ -291,10 +359,14 @@ class SMBConnection():
         """
 
         if self.getDialect() == smb.SMB_DIALECT:
-            pathName = string.replace(pathName, '/', '\\')
+            _, flags2 = self._SMBConnection.get_flags()
+
+            pathName = pathName.replace('/', '\\')
+            pathName = pathName.encode('utf-16le') if flags2 & smb.SMB.FLAGS2_UNICODE else pathName
+
             ntCreate = smb.SMBCommand(smb.SMB.SMB_COM_NT_CREATE_ANDX)
             ntCreate['Parameters'] = smb.SMBNtCreateAndX_Parameters()
-            ntCreate['Data']       = smb.SMBNtCreateAndX_Data()
+            ntCreate['Data']       = smb.SMBNtCreateAndX_Data(flags=flags2)
             ntCreate['Parameters']['FileNameLength']= len(pathName)
             ntCreate['Parameters']['AccessMask']    = desiredAccess
             ntCreate['Parameters']['FileAttributes']= fileAttributes
@@ -306,6 +378,9 @@ class SMBConnection():
             ntCreate['Parameters']['CreateFlags']   = 0x16
             ntCreate['Data']['FileName'] = pathName
 
+            if flags2 & smb.SMB.FLAGS2_UNICODE:
+                ntCreate['Data']['Pad'] = 0x0
+
             if createContexts is not None:
                 LOG.error("CreateContexts not supported in SMB1")
 
@@ -330,10 +405,14 @@ class SMBConnection():
         """
 
         if self.getDialect() == smb.SMB_DIALECT:
-            pathName = string.replace(pathName, '/', '\\')
+            _, flags2 = self._SMBConnection.get_flags()
+
+            pathName = pathName.replace('/', '\\')
+            pathName = pathName.encode('utf-16le') if flags2 & smb.SMB.FLAGS2_UNICODE else pathName
+
             ntCreate = smb.SMBCommand(smb.SMB.SMB_COM_NT_CREATE_ANDX)
             ntCreate['Parameters'] = smb.SMBNtCreateAndX_Parameters()
-            ntCreate['Data']       = smb.SMBNtCreateAndX_Data()
+            ntCreate['Data']       = smb.SMBNtCreateAndX_Data(flags=flags2)
             ntCreate['Parameters']['FileNameLength']= len(pathName)
             ntCreate['Parameters']['AccessMask']    = desiredAccess
             ntCreate['Parameters']['FileAttributes']= fileAttributes
@@ -345,6 +424,9 @@ class SMBConnection():
             ntCreate['Parameters']['CreateFlags']   = 0x16
             ntCreate['Data']['FileName'] = pathName
 
+            if flags2 & smb.SMB.FLAGS2_UNICODE:
+                ntCreate['Data']['Pad'] = 0x0
+
             if createContexts is not None:
                 LOG.error("CreateContexts not supported in SMB1")
 
@@ -375,22 +457,49 @@ class SMBConnection():
             raise SessionError(e.get_error_code())
 
 
-    def readFile(self, treeId, fileId, offset = 0, bytesToRead = None):
+    def readFile(self, treeId, fileId, offset = 0, bytesToRead = None, singleCall = True):
         """
         reads data from a file
 
         :param HANDLE treeId: a valid handle for the share where the file is to be read
         :param HANDLE fileId: a valid handle for the file to be read
         :param integer offset: offset where to start reading the data
-        :param integer bytesToRead: amount of bytes to read. If None, it will read Dialect['MaxBufferSize'] bytes.
+        :param integer bytesToRead: amount of bytes to attempt reading. If None, it will attempt to read Dialect['MaxBufferSize'] bytes.
+        :param boolean singleCall: If True it won't attempt to read all bytesToRead. It will only make a single read call
 
-        :return: the data read, if not raises a SessionError exception.
+        :return: the data read, if not raises a SessionError exception. Length of data read is not always bytesToRead
         """
-        try:
-            return self._SMBConnection.read_andx(treeId, fileId, offset, bytesToRead)
-        except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+        finished = False
+        data = ''
+        maxReadSize = self._SMBConnection.getIOCapabilities()['MaxReadSize']
+        remainingBytesToRead = bytesToRead
+        while not finished:
+            if remainingBytesToRead > maxReadSize:
+                toRead = maxReadSize
+            else:
+                toRead = remainingBytesToRead
+            try:
+                bytesRead = self._SMBConnection.read_andx(treeId, fileId, offset, toRead)
+            except (smb.SessionError, smb3.SessionError), e:
+                if e.get_error_code() == nt_errors.STATUS_END_OF_FILE:
+                    toRead = ''
+                    break
+                else:
+                    raise SessionError(e.get_error_code())
 
+            data += bytesRead
+            if len(data) >= bytesToRead:
+                finished = True
+            elif len(bytesRead) == 0:
+                # End of the file achieved.
+                finished = True
+            elif singleCall is True:
+                finished = True
+            else:
+                offset += len(bytesRead)
+                remainingBytesToRead -= len(bytesRead)
+
+        return data
 
     def closeFile(self, treeId, fileId):
         """
@@ -422,6 +531,25 @@ class SMBConnection():
         except (smb.SessionError, smb3.SessionError), e:
             raise SessionError(e.get_error_code())
 
+    def queryInfo(self, treeId, fileId):
+        """
+        queries basic information about an opened file/directory
+
+        :param HANDLE treeId: a valid handle for the share where the file is to be opened
+        :param HANDLE fileId: a valid handle for the file/directory to be closed
+
+        :return: a smb.SMBQueryFileBasicInfo structure.  raises a SessionError exception if error.
+
+        """
+        try:
+            if self.getDialect() == smb.SMB_DIALECT:
+                res = self._SMBConnection.query_file_info(treeId, fileId)
+            else:
+                res = self._SMBConnection.queryInfo(treeId, fileId)
+            return smb.SMBQueryFileStandardInfo(res)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
     def createDirectory(self, shareName, pathName ):
         """
         creates a directory
@@ -525,15 +653,14 @@ class SMBConnection():
 
         :param HANDLE treeId: a valid handle for the share where the pipe resides
         :param HANDLE fileId: a valid handle for the pipe
-        :param integer bytestToRead: amount of data to read
-        :param boolean waitAnswer: whether or not to wait for an answer
+        :param integer bytesToRead: amount of data to read
 
         :return: None, raises a SessionError exception if error.
 
         """
 
         try:
-            return self.readFile(treeId, fileId, bytesToRead = bytesToRead)
+            return self.readFile(treeId, fileId, bytesToRead = bytesToRead, singleCall = True)
         except (smb.SessionError, smb3.SessionError), e:
             raise SessionError(e.get_error_code())
 
@@ -550,11 +677,11 @@ class SMBConnection():
 
         """
         try:
-	    if shareAccessMode is None:
-		# if share access mode is none, let's the underlying API deals with it
-            	return self._SMBConnection.retr_file(shareName, pathName, callback)
-	    else:
-            	return self._SMBConnection.retr_file(shareName, pathName, callback, shareAccessMode = shareAccessMode)
+            if shareAccessMode is None:
+                # if share access mode is none, let's the underlying API deals with it
+                return self._SMBConnection.retr_file(shareName, pathName, callback)
+            else:
+                return self._SMBConnection.retr_file(shareName, pathName, callback, shareAccessMode=shareAccessMode)
         except (smb.SessionError, smb3.SessionError), e:
             raise SessionError(e.get_error_code())
 
@@ -570,11 +697,11 @@ class SMBConnection():
 
         """
         try:
-	    if shareAccessMode is None:
-		# if share access mode is none, let's the underlying API deals with it
-            	return self._SMBConnection.stor_file(shareName, pathName, callback)
-	    else: 
-            	return self._SMBConnection.stor_file(shareName, pathName, callback, shareAccessMode)
+            if shareAccessMode is None:
+                # if share access mode is none, let's the underlying API deals with it
+                return self._SMBConnection.stor_file(shareName, pathName, callback)
+            else:
+                return self._SMBConnection.stor_file(shareName, pathName, callback, shareAccessMode)
         except (smb.SessionError, smb3.SessionError), e:
             raise SessionError(e.get_error_code())
 
