@@ -1,5 +1,5 @@
 #!/usr/local/bin/python2.7
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -47,23 +47,20 @@
 #
 #
 import sys
-import os
-import cmd
 import argparse
-import time
-import ntpath
 import logging
+import socket
 from threading import Thread, Event
 from Queue import Queue
 from time import sleep
 
-from impacket import version, ntlm
-from impacket.smbconnection import *
+from impacket.examples import logger
+from impacket import version
+from impacket.smbconnection import SessionError
 from impacket.dcerpc.v5 import transport, wkst, srvs, samr
 from impacket.dcerpc.v5.ndr import NULL
 from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket.nt_errors import STATUS_MORE_ENTRIES
-from impacket.examples import logger
 
 machinesAliveQueue = Queue()
 machinesDownQueue = Queue()
@@ -119,6 +116,7 @@ class USERENUM:
         self.__targets = dict()
         self.__filterUsers = None
         self.__targetsThreadEvent = None
+        self.__targetsThread = None
         self.__maxConnections = int(options.max_connections)
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
@@ -155,7 +153,7 @@ class USERENUM:
             while status == STATUS_MORE_ENTRIES:
                 try:
                     resp = samr.hSamrEnumerateUsersInDomain(dce, domainHandle, samr.USER_WORKSTATION_TRUST_ACCOUNT, enumerationContext = enumerationContext)
-                except Exception, e:
+                except DCERPCException, e:
                     if str(e).find('STATUS_MORE_ENTRIES') < 0:
                         raise 
                     resp = e.get_packet()
@@ -426,6 +424,8 @@ class USERENUM:
 # Process command-line arguments.
 if __name__ == '__main__':
     print version.BANNER
+    # Init the example's logger theme
+    logger.init()
 
     parser = argparse.ArgumentParser()
 
