@@ -1,5 +1,5 @@
 #!/usr/local/bin/python2.7
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -16,18 +16,19 @@
 #
 
 import sys
-import string
 import time
 import logging
-from impacket.examples import logger 
-from impacket import smb, version, smb3, nt_errors, version
+import argparse
+import cmd
+import os
+
+from impacket.examples import logger
+from impacket import version
 from impacket.dcerpc.v5 import samr, transport, srvs
 from impacket.dcerpc.v5.dtypes import NULL
 from impacket.smbconnection import *
-import argparse
-import ntpath
-import cmd
-import os
+
+
 # If you wanna have readline like functionality in Windows, install pyreadline
 try:
   import pyreadline as readline
@@ -45,6 +46,7 @@ class MiniImpacketShell(cmd.Cmd):
         self.pwd = ''
         self.share = None
         self.loggedIn = True
+        self.last_output = None
         self.completion = []
 
     def emptyline(self):
@@ -83,6 +85,7 @@ class MiniImpacketShell(cmd.Cmd):
  shares - list available shares
  use {sharename} - connect to an specific share
  cd {path} - changes the current directory to {path}
+ lcd {path} - changes the current local directory to {path}
  pwd - shows current remote directory
  password - changes the user password, the new password will be prompted for input
  ls {wildcard} - lists all the files in the current directory
@@ -108,7 +111,7 @@ class MiniImpacketShell(cmd.Cmd):
         dce = rpctransport.get_dce_rpc()
         dce.connect()                     
         dce.bind(samr.MSRPC_UUID_SAMR)
-        resp = samr.hSamrUnicodeChangePasswordUser2(dce, '\x00', self.username, self.password, newPassword, self.lmhash, self.nthash)
+        samr.hSamrUnicodeChangePasswordUser2(dce, '\x00', self.username, self.password, newPassword, self.lmhash, self.nthash)
         self.password = newPassword
         self.lmhash = None
         self.nthash = None
@@ -245,7 +248,7 @@ class MiniImpacketShell(cmd.Cmd):
             logging.error("No connection open")
             return
         self.smb.logoff()
-        del(self.smb)
+        del self.smb
         self.share = None
         self.smb = None
         self.tid = None
@@ -319,20 +322,22 @@ class MiniImpacketShell(cmd.Cmd):
         self.pwd = ntpath.normpath(self.pwd)
         # Let's try to open the directory to see if it's valid
         try:
-            fid = self.smb.openFile(self.tid, self.pwd)
+            fid = self.smb.openFile(self.tid, self.pwd, creationOption = FILE_DIRECTORY_FILE \
+                                    , desiredAccess = FILE_READ_DATA | FILE_LIST_DIRECTORY \
+                                    , shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE \
+                                    )
             self.smb.closeFile(self.tid,fid)
-            self.pwd = oldpwd
-            logging.error("Invalid directory")
-        except SessionError, e:
-            if e.getErrorCode() == nt_errors.STATUS_FILE_IS_A_DIRECTORY: 
-               pass
-            else:
-               self.pwd = oldpwd
-               raise
-        except Exception, e:
+        except SessionError:
             self.pwd = oldpwd
             raise
 
+    def do_lcd(self, s):
+        print s
+        if s == '':
+           print os.getcwd()
+        else:
+           os.chdir(s)
+
     def do_pwd(self,line):
         if self.loggedIn is False:
             logging.error("Not logged in")
@@ -437,6 +442,8 @@ class MiniImpacketShell(cmd.Cmd):
         self.do_logoff(line)
 
 def main():
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
     parser = argparse.ArgumentParser(add_help = True, description = "SMB client implementation.")
 
@@ -465,6 +472,11 @@ def main():
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
+
     if domain is None:
         domain = ''
     
